package dev.flyfish.framework.approval.controller;

import dev.flyfish.framework.approval.domain.ApprovalDomain;
import dev.flyfish.framework.approval.domain.record.ApproveRecord;
import dev.flyfish.framework.approval.domain.record.ApproveRecordListVo;
import dev.flyfish.framework.approval.domain.record.ApproveRecordQo;
import dev.flyfish.framework.approval.domain.submit.ApprovalSubmitDto;
import dev.flyfish.framework.approval.domain.todo.ApprovalDto;
import dev.flyfish.framework.approval.domain.todo.ApprovalListVo;
import dev.flyfish.framework.approval.service.ApproveRecordService;
import dev.flyfish.framework.approval.service.ModuleDelegateService;
import dev.flyfish.framework.bean.Result;
import dev.flyfish.framework.configuration.annotations.CurrentUser;
import dev.flyfish.framework.configuration.annotations.PagedQuery;
import dev.flyfish.framework.domain.base.Vo;
import dev.flyfish.framework.domain.po.User;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import java.util.List;

/**
 * 审批专用controller
 *
 * @author wangyu
 */
@RestController
@RequestMapping("approves")
public class ApprovalController {

    @Resource
    private ApproveRecordService approveRecordService;
    @Resource
    private ModuleDelegateService moduleDelegateService;

    /**
     * 查询视图
     *
     * @return 结果
     */
    @GetMapping("records")
    public Mono<Result<List<Vo<ApproveRecord>>>> views(@PagedQuery ApproveRecordQo qo) {
        return approveRecordService.getPageList(qo)
                .map(Result::accept)
                .map(result -> result.mapList((ApproveRecord item) -> new ApproveRecordListVo().from(item)));
    }

    /**
     * 查询待办列表
     *
     * @param qo   查询实体
     * @param user 用户
     * @return 结果
     */
    @GetMapping("todos")
    public Mono<Result<List<Vo<ApproveRecord>>>> todos(@PagedQuery ApproveRecordQo qo, @CurrentUser User user) {
        qo.setApprover(user.getId());
        qo.setApproved(false);
        return approveRecordService.getPageList(qo)
                .map(Result::accept)
                .map(result -> result.mapList((ApproveRecord item) -> new ApprovalListVo().from(item)));
    }

    /**
     * 提交审批
     * 需要指定提交候选人，按顺序审批
     *
     * @return 结果
     */
    @PostMapping("submit")
    public Mono<Result<ApprovalDomain>> submit(@Valid @RequestBody ApprovalSubmitDto body) {
        return moduleDelegateService.submit(body).map(Result::ok)
                .onErrorResume(e -> Mono.just(Result.error(e.getMessage())));
    }


    /**
     * 审批数据
     *
     * @param approval 数据
     * @return 结果
     */
    @PostMapping("approval")
    public Mono<Result<ApprovalDomain>> approval(@RequestBody ApprovalDto approval) {
        return moduleDelegateService.approval(approval).map(Result::ok);
    }
}
