package dev.flyfish.framework.domain.authorized.advanced;

import dev.flyfish.framework.domain.authorized.AbstractAuthorizedQo;
import dev.flyfish.framework.domain.authorized.AuthorizedUserDetails;
import dev.flyfish.framework.enums.UserType;
import dev.flyfish.framework.query.QueryDefinition;
import dev.flyfish.framework.query.Queries;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.BooleanUtils;

/**
 * 拥有归属权的授权查询
 *
 * @param <T> 泛型
 * @author wangyu
 * 将会额外查询归属者的内容
 */
@Getter
@Setter
public class OwnedAuthorizedQo<T extends OwnedAuthorizedDomain> extends AbstractAuthorizedQo<T> {

    @Override
    public QueryDefinition queryBuilder() {
        // 超级管理员拥有查看所有草稿的权限
        if (user.getType() == UserType.SUPER_ADMIN) {
            return super.queryBuilder().mutate().and(this.withPublished());
        }
        // 查询草稿，只查询自己的
        if (BooleanUtils.isFalse(published)) {
            return super.queryBuilder()
                    .mutate()
                    .and(this.withPublished())
                    // 此处原意为使用用户id匹配owners列中的id，修改后框架会自动判定并解包，存在耦合性
                    .and(Queries
                            .where(T::getCreatorId).eq(user.getId())
                            .or(T::getOwners).hasId(user.getId()));
        }
        // 普通查询，根据权限配置查询
        return super.queryBuilder()
                .mutate()
                .and(this.withPublished())
                .and(() -> Queries.where(T::getAuthorizeId).in(authorizeIds())
                        .or(Queries.where(T::getCreatorId).eq(user.getId())
                                .and(T::getAuthorizeId).in(((AuthorizedUserDetails) user).getVisibleDeparts()))
                        .or(Queries.where(T::getOwners).hasId(user.getId()))
                );
    }
}
