package dev.flyfish.framework.generation.code.strategy;

import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.generation.CodeRuleStrategy;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 流水号生成策略
 *
 * @author wangyu
 */
@Component
public class TransactionCodeRuleStrategy implements CodeRuleStrategy {

    // 每天凌晨自动清零
    private final AtomicInteger atomicInteger = new AtomicInteger(0);

    /**
     * 每天0点自动清零
     */
    @Scheduled(cron = "0 0 0 * * ?")
    public void clear() {
        atomicInteger.set(0);
    }

    /**
     * 生成代码
     *
     * @param domain 实体
     * @return 结果
     */
    @Override
    public String generate(Domain domain) {
        String date = DateFormatUtils.format(new Date(), "yyyyMMddHHmmss");
        String flow = StringUtils.leftPad(String.valueOf(atomicInteger.incrementAndGet()), 4, "0");
        return date + flow;
    }
}
