package dev.flyfish.framework.generation.property;

import dev.flyfish.framework.annotations.Generation;
import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.utils.ReflectionUtils;
import dev.flyfish.framework.utils.UUIDUtils;
import org.apache.commons.lang3.reflect.FieldUtils;
import org.springframework.data.util.CastUtils;

import java.lang.reflect.Field;
import java.util.Map;
import java.util.Optional;

/**
 * 生成策略工具类
 */
public final class Generations {

    private static final GenerateStrategy<String> UUID = domain -> UUIDUtils.uuid();

    private static final GenerateStrategy<Object> NO_OP = domain -> null;

    private static final Map<Generation.Strategy, GenerateStrategy<?>> STRATEGIES = Map.of(
            Generation.Strategy.UUID, UUID
    );

    public static <T> GenerateStrategy<T> of(Generation generation) {
        return CastUtils.cast(STRATEGIES.getOrDefault(generation.value(), NO_OP));
    }

    public static void apply(Domain domain) {
        for (Field field : FieldUtils.getFieldsWithAnnotation(domain.getClass(), Generation.class)) {
            ReflectionUtils.makeAccessible(field);
            GenerateStrategy<?> strategy = of(field.getAnnotation(Generation.class));
            Optional.ofNullable(strategy.generate(domain)).ifPresent(value -> setValue(field, domain, value));
        }
    }

    private static void setValue(Field field, Object obj, Object value) {
        try {
            ReflectionUtils.makeAccessible(field);
            field.set(obj, value);
        } catch (Exception ignored) {
        }
    }
}
