package dev.flyfish.framework.query;

import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.domain.base.Qo;
import dev.flyfish.framework.domain.base.SimpleQo;
import dev.flyfish.framework.query.spi.adaptor.CriteriaAdaptor;

/**
 * 可构建的
 *
 * @author wangyu
 */
public interface QueryDefinition {

    /**
     * 构建结果
     *
     * @param adaptor 查询适配器
     * @param <T>     泛型
     * @return 构建结果
     */
    <T> T build(CriteriaAdaptor<T> adaptor);

    /**
     * 包装快捷方法
     *
     * @param <D> 实体类型泛型
     * @return 结果
     */
    default <D extends Domain> Qo<D> wrap() {
        return new SimpleQo<>(this);
    }

    /**
     * 修改，此修改会直接接着查询条件进行拼接
     *
     * @return 修改句柄
     */
    QueryMutation mutate();

    /**
     * 会直接添加嵌套查询到当前查询末尾
     *
     * @return 修改句柄
     */
    QueryMutation within();

    /**
     * 判断查询条件是否为空
     *
     * @return 结果
     */
    boolean isEmpty();
}
