package dev.flyfish.framework.query.holder;

import dev.flyfish.framework.query.spi.adaptor.CriteriaAdaptor;

import java.util.function.BinaryOperator;
import java.util.function.Function;

/**
 * 查询链保持器，将所有拼接工作后置，可以做检查和处理
 *
 * @author wangyu
 */
public interface QueryChainHolder<C> {

    /**
     * 等待下一步消费
     * 如果不调用with，该状态将一直保持，可被其他link调用替换
     * 在调用with后，自动消费，并完成拼接
     *
     * @param operator 操作方法
     * @return 本身
     */
    QueryChainHolder<C> link(Function<CriteriaAdaptor<C>, BinaryOperator<C>> operator);

    /**
     * 拼接条件
     *
     * @param criteria 条件
     */
    void with(Function<CriteriaAdaptor<C>, C> criteria);

    /**
     * 触发动作，得到最终的
     *
     * @return 本身
     */
    C get(CriteriaAdaptor<C> adaptor);

    /**
     * 判断条件是否为空
     *
     * @return 结果
     */
    boolean isEmpty();
}
