package dev.flyfish.framework.mongodb.config.callback;

import dev.flyfish.framework.auditor.callback.ReactiveAuditorCallback;
import dev.flyfish.framework.domain.base.Domain;
import org.reactivestreams.Publisher;
import org.springframework.data.mongodb.core.mapping.event.ReactiveBeforeConvertCallback;
import org.springframework.lang.NonNull;

public class MongoReactiveAuditorMongoCallback extends ReactiveAuditorCallback implements ReactiveBeforeConvertCallback<Domain> {

    /**
     * 实体回调方法，用于在域对象转换为持久化对象之前调用。该方法可以返回相同的域对象，也可以返回经过修改的域对象，并使用 {@link Publisher} 发布要持久化的对象。
     *
     * @param entity     要保存的领域对象。
     * @param collection 集合名称 (可能指数据库中的集合概念)。
     * @return 发布要持久化的领域对象的 {@link Publisher} 对象。
     */
    @Override
    @NonNull
    public Publisher<Domain> onBeforeConvert(@NonNull Domain entity, @NonNull String collection) {
        // 执行auditor
        return audit(entity);
    }
}
