package dev.flyfish.framework.relational.mapping;

import org.springframework.data.annotation.Transient;

import java.lang.annotation.*;

/**
 * 数据库关联信息
 *
 * @author wangyu
 */
@Target({ElementType.FIELD})
@Retention(RetentionPolicy.RUNTIME)
@Documented
@Transient
public @interface Association {

    /**
     * @return 关联在本对象上的字段名称
     * 可不指定。不指定时，会自动取得当前对象上 @Reference 注解的字段
     * <p>
     * 如当前表的detail_id字段对应detail表的id字段，则该值为 detail_id
     */
    String field() default "";

    /**
     * @return 关联在外部对象上的字段名称，注意，禁止循环依赖
     * 可不指定。不指定时，会自动取得目标类型中 @Reference 注解的字段
     * <p>
     * 如当前表不存在目标表detail的关联id，目标表关联本实体的id为main_id，则该值为main_id。
     * 与field互斥，同时指定永远都以field为主
     */
    String foreignField() default "";

    /**
     * 关系表，针对多对多的情况
     * 配置了该项的情况下，field和foreignField都为该表上的字段
     *
     * @return 结果
     */
    String relationTable() default "";
}
