package dev.flyfish.framework.dict.service;


import dev.flyfish.framework.dict.domain.Dictionary;
import dev.flyfish.framework.dict.repository.DictionaryRepository;
import dev.flyfish.framework.service.impl.BaseReactiveServiceImpl;
import dev.flyfish.framework.utils.ReactiveRedisOperations;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import jakarta.annotation.Resource;

/**
 * 字典表服务
 *
 * @author wangyu
 */
@Service
public class DictionaryService extends BaseReactiveServiceImpl<Dictionary> {

    private final String DICT_KEY = "dict-";

    @Resource
    private ReactiveRedisOperations reactiveRedisOperations;

    /**
     * 通过code查询
     *
     * @return 结果
     */
    public Mono<Dictionary> getByCode(String code) {
        String cacheKey = DICT_KEY + code;
        return reactiveRedisOperations.hasKey(cacheKey)
                .flatMap(hasKey -> {
                    if (hasKey) {
                        return reactiveRedisOperations.get(cacheKey).map(obj -> (Dictionary) obj);
                    }
                    DictionaryRepository repository = this.getRepository();
                    return repository.findByCode(code);
                });
    }
}
