package dev.flyfish.framework.dict;

import dev.flyfish.framework.configuration.boot.FlyfishApplication;
import dev.flyfish.framework.dict.annotations.EnableDictProcess;
import dev.flyfish.framework.dict.domain.Dictionary;
import dev.flyfish.framework.dict.domain.DictionaryValue;
import dev.flyfish.framework.dict.repository.DictionaryRepository;
import dev.flyfish.framework.r2dbc.config.R2dbcDataConfig;
import dev.flyfish.framework.utils.JacksonUtil;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Import;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.Arrays;

@ExtendWith(SpringExtension.class)
@SpringBootTest(classes = FlyfishDictTest.class)
@FlyfishApplication(scanBasePackages = "dev.flyfish.framework")
@Import(R2dbcDataConfig.class)
@EnableDictProcess
@Slf4j
public class FlyfishDictTest {

    @Resource
    private DictionaryRepository dictionaryRepository;

    @Test
    public void test() {
        Dictionary dictionary = new Dictionary();
        dictionary.setName("测试字典");
        dictionary.setCode("test");
        dictionary.setDescription("测试用");
        dictionary.setValues(Arrays.asList(
                DictionaryValue.of("值1", "1"),
                DictionaryValue.of("值2", "2")
        ));

        Dictionary saved = dictionaryRepository.insert(dictionary)
                .doOnNext(this::log)
                .block();
    }

    private void log(Object data) {
        log.info(JacksonUtil.toPrettyJson(data));
    }
}
