package dev.flyfish.framework.handler;

import dev.flyfish.framework.bean.Result;
import dev.flyfish.framework.service.AuthenticationAuditor;
import dev.flyfish.framework.utils.AuthenticationMessages;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.server.WebFilterExchange;
import org.springframework.security.web.server.authentication.ServerAuthenticationFailureHandler;
import reactor.core.publisher.Mono;

/**
 * 基于json的登录失败包装，详见Spring Security
 *
 * @author wangyu
 */
public class JsonAuthenticationFailureHandler implements ServerAuthenticationFailureHandler {

    // 数据块工厂
    private final AuthenticationAuditor authenticationAuditor;

    public JsonAuthenticationFailureHandler(AuthenticationAuditor authenticationAuditor) {
        this.authenticationAuditor = authenticationAuditor;
    }

    /**
     * 失败时返回具体失败原因
     *
     * @param webFilterExchange 请求数据
     * @param exception         异常
     * @return 结果
     */
    @Override
    public Mono<Void> onAuthenticationFailure(WebFilterExchange webFilterExchange, AuthenticationException exception) {
        ServerHttpResponse response = webFilterExchange.getExchange().getResponse();
        response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
        // 记录错误次数，错误日志等，最后写入
        return authenticationAuditor.error(webFilterExchange, exception).then(write(response, exception));
    }

    /**
     * 写入
     *
     * @param response  响应
     * @param exception 异常
     * @return 结果
     */
    private Mono<Void> write(ServerHttpResponse response, AuthenticationException exception) {
        return response.writeWith(authenticationAuditor.transform(Result.error(AuthenticationMessages.message(exception))));
    }
}
